﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ComponentModel.DataAnnotations;
using System.Runtime.Serialization;
using System.ComponentModel;

using IdeaBlade.EntityModel;

namespace DomainModel {

  // POCO types must be "known" types, and must be serializable.
  // Indicating that this is a "known" type allows instances of the type to be queried and saved in an n-tier environment, such as Silverlight.
  // There are several ways to identify the type as a "known" type to DevForce:
  // (1) Decorate a property (or multiple properties) with the Key attribute; or
  // (2) Decorate the type with the KnownType attribute; or
  // (3) Implement the IKnownType interface; or
  // (4) Implement the IKnownTypeProvider interface
  //
  // The class must also be serializable:  
  //   1) Mark the class and members with the DataContract and DataMember attributes, or
  //   2) The public properties of the class will be serializable if the class has a public parameterless constructor.  This technique is
  //      used here.

  public class State: IKnownType, IHasPocoEntityAspect, INotifyPropertyChanged {
    public State() {
    }

    [Key]
    public string Abbrev {
      get {
        return _abbrev;
      }
      set {
        _abbrev = value;
        OnPropertyChanged("Abbrev");
      }
    }

    public string Name {
      get {
        return _name;
      }
      set {
        _name = value;
        OnPropertyChanged("Name");
      }
    }

    public bool Lower49 {
      get {
        return _lower49;
      }
      set {
        _lower49 = value;
        OnPropertyChanged("Lower49");
      }
    }

    #region Private Fields
    string _abbrev;
    string _name;
    bool _lower49;
    #endregion Private Fields

    #region IHasPocoEntityAspect Members

    [Display(AutoGenerateField=false)]
    [IgnoreDataMember]  //We don't want to serialize EntityAspect here as EntityAspect is not serializable
    public IdeaBlade.EntityModel.EntityAspect EntityAspect {
      get;
      set;
    }

    #endregion

    #region INotifyPropertyChanged Members

    /// <summary>
    /// This interface implementation is needed if you want EntityManager to automatically listen
    /// to any property change.
    /// </summary>

    public event PropertyChangedEventHandler PropertyChanged;

    protected void OnPropertyChanged(String propertyName) {
      var handler = PropertyChanged;
      if (handler != null) {
        var args = new PropertyChangedEventArgs(propertyName);
        handler(this, args);
      }
    }

    #endregion
  }
}
